@@ -27,8 +27,9 @@ import unittest
 from ConfigParser import ConfigParser
 from UserDict import UserDict
 from UserList import UserList
+from functools import update_wrapper
 from inspect import getargs, isclass, isfunction, ismethod
-from logging import FileHandler, Formatter, StreamHandler
+from logging import Formatter, StreamHandler
 from operator import itemgetter, attrgetter
 from string import letters
 
@@ -59,6 +60,125 @@ class MainError(Error):
 class ParameterError(Error):
     pass
 
+def fmtsec(seconds):
+    if seconds < 0:
+        return '-' + self.seconds(-seconds)
+
+    prefixes = " munp"
+    powers = range(0, 3 * len(prefixes) + 1, 3)
+
+    prefix = ''
+    for power, prefix in zip(powers, prefixes):
+        if seconds >= pow(10.0, -power):
+            seconds *= pow(10.0, power)
+            break
+
+    formats = [
+        (1e9, "%.4g"),
+        (1e6, "%.0f"),
+        (1e5, "%.1f"),
+        (1e4, "%.2f"),
+        (1e3, "%.3f"),
+        (1e2, "%.4f"),
+        (1e1, "%.5f"),
+        (1e0, "%.6f"),
+    ]
+
+    for threshold, format in formats:
+        if seconds >= threshold:
+            break
+
+    format += " %ss"
+    return format % (seconds, prefix)
+
+
+class Profiler(object):
+
+    def __init__(self, stdout=None, anonymous=False, count=1000, repeat=3):
+        self.stdout = stdout is None and sys.stdout or stdout
+        self.anonymous = anonymous
+        self.count = count
+        self.repeat = repeat
+
+    def wrap(self, wrapper, wrapped):
+        update_wrapper(wrapper, wrapped)
+
+        if self.anonymous or self.isanon(wrapped.func_name):
+            return wrapper()
+        else:
+            return wrapper
+
+    def isanon(self, func_name):
+        return func_name.startswith("__profiler_") or \
+            func_name == "anonymous"
+
+    def deterministic(self, func):
+        from pstats import Stats
+
+        try:
+            from cProfile import Profile
+        except ImportError:
+            from profile import Profile
+        profiler = Profile()
+
+        def wrapper(*args, **kwargs):
+            self.stdout.write("===> Profiling %s:\n" % func.func_name)
+            profiler.runcall(func, *args, **kwargs)
+            stats = Stats(profiler, stream=self.stdout)
+            stats.strip_dirs().sort_stats(-1).print_stats()
+
+        return self.wrap(wrapper, func)
+
+    def statistical(self, func):
+        try:
+            from timeit import default_timer as timer
+        except ImportError:
+            from time import time as timer
+
+        def timeit(func, *args, **kwargs):
+            cumulative = 0
+            for i in range(self.count):
+                start = timer()
+                func(*args, **kwargs)
+                stop = timer()
+                cumulative += stop - start
+
+            return cumulative
+
+        def repeat(func, *args, **kwargs):
+            return [timeit(func, *args, **kwargs) for i in range(self.repeat)]
+
+        def wrapper(*args, **kwargs):
+            self.stdout.write("===> Profiling %s: " % func.func_name)
+            result = min(repeat(func, *args, **kwargs))
+            self.stdout.write("%d loops, best of %d: %s per loop\n" % (
+                self.count, self.repeat, fmtsec(result/self.count)))
+
+        return self.wrap(wrapper, func)
+
+    __call__ = deterministic
+
+class FileHandler(logging.FileHandler):
+
+    def close(self):
+        """Override close().
+        
+        We leave the file open because the application may have
+        multiple threads that still need to write to it. Python
+        should GC the fd when it goes out of scope, anyway.
+        """
+        pass
+
+class NullHandler(logging.Handler):
+    """A blackhole handler.
+
+    NullHandler simply ignores all messages it receives.
+    """
+
+    def emit(self, record):
+        """Ignore the record."""
+        pass
+
 class CLILogger(logging.Logger):
     """Provide extra configuration smarts for loggers.
 
@@ -253,15 +373,16 @@ class EnvironParameterHandler(ParameterHandler):
     delim = '_'
 
     def __init__(self, environ):
-        self.environ = environ
+        self.environ = environ or os.environ
 
     def handle(self, app, parameters):
         super(EnvironParameterHandler, self).handle(app, parameters)
 
-        if [x for x in parameters]:
-            names = [self.get_param_name(x) for x in parameters]
+        pvars = [x for x in parameters if getattr(x, "var_name", None)]
+        if pvars:
             app.epilog += "Environment variables:\n"
-            app.epilog += '\n'.join(names)
+            names = [self.get_param_name(x) for x in pvars]
+            app.epilog += '\n'.join('  %s' % n for n in names)
 
     def get_param_name(self, parameter):
         """Convert parameter name to something useful. 
@@ -281,21 +402,28 @@ class EnvironParameterHandler(ParameterHandler):
         if value is not None:
             parameter.value = value
 
+class OptionParser(optparse.OptionParser):
+
+    def format_epilog(self, formatter):
+        return "\n%s\n" % self.epilog
+
 class CLIParameterHandler(ParameterHandler):
     delim = '-'
     cmp = staticmethod(lambda x, y: \
             cmp(getattr(x, "short", x.name), getattr(y, "short", y.name)))
+    parser_factory = OptionParser
 
     def __init__(self, argv):
         self.argv = argv
 
     def handle(self, app, parameters):
-        self.parser = optparse.OptionParser(
+        self.parser = self.parser_factory(
                 usage=app.usage,
                 version='',     # XXX: this would be nice
                 description='', # same here
                 epilog=app.epilog,
                 )
+        self.parser.process_default_values = False
 
         # XXX: we'll need to map option names to parameters here...
         param_map = {}
@@ -366,7 +494,7 @@ class CommandLineApp(object):
 
     def __init__(self, main, config_file=None, argv=None, env={},
             exit_after_main=True, stdin=None, stdout=None,
-            stderr=None, epilog=""):
+            stderr=None, epilog="", profiler=None):
         self.main = main
         self.config_file = config_file
         self.argv = argv
@@ -378,6 +506,10 @@ class CommandLineApp(object):
         self.args = []
         self.epilog = epilog
 
+        if profiler is None:
+            profiler = Profiler(self.stderr, anonymous=True)
+        self.profiler = profiler
+
         self.params = self.param_factory("root")
         self.param_handlers = []
 
@@ -482,7 +614,7 @@ class LoggingApp(CommandLineApp):
     None.
     """
 
-    def __init__(self, main, stream=None, logfile=None,
+    def __init__(self, main, stream=sys.stdout, logfile=None,
             message_format="%(message)s", 
             date_format="%(asctime)s %(message)s", **kwargs):
         self.logfile = logfile
@@ -505,29 +637,34 @@ class LoggingApp(CommandLineApp):
         # Create logger.
         logging.setLoggerClass(CLILogger)
         self.log = logging.getLogger(self.name)
-        
-        # Create handlers.
-        stream_handler = StreamHandler(self.stream)
-        handler = stream_handler
-        if self.logfile is not None:
-            file_handler = FileHandler(self.logfile)
-            handler = file_handler
 
         # Create formatters.
         message_formatter = Formatter(self.message_format)
         date_formatter = Formatter(self.date_format)
         verbose_formatter = Formatter()
-        formatter = message_formatter
+        self.formatter = message_formatter
 
-        handler.setFormatter(formatter)
         self.log.setLevel(self.log.default_level)
-        self.log.addHandler(handler)
 
     def pre_run(self):
         """Configure logging before running the app."""
         super(LoggingApp, self).pre_run()
         self.log.setLevel(opts=self.params)
 
+        logfile = getattr(self.params, "logfile", None) or self.logfile
+        if logfile is not None:
+            file_handler = FileHandler(logfile)
+            file_handler.setFormatter(self.formatter)
+            self.log.addHandler(file_handler)
+        elif self.stream is not None:
+            stream_handler = StreamHandler(self.stream)
+            stream_handler.setFormatter(self.formatter)
+            self.log.addHandler(stream_handler)
+
+        # The null handler simply drops all messages.
+        if not self.log.handlers:
+            self.log.addHandler(NullHandler())
+
 class DaemonizingApp(LoggingApp):
     """A command-line application that knows how to daemonize.
     """
@@ -547,19 +684,14 @@ class DaemonizingApp(LoggingApp):
                 action="store_true")
         self.add_param("user", None, "change to USER[:GROUP] after daemonizing")
         self.add_param("pidfile", None, "write PID to PIDFILE after daemonizing")
+        self.add_param("logfile", None, "write logs to LOGFILE")
 
     def daemonize(self):
         """Daemonize the application.
         
-        If the 'daemonize' parameter is not True, daemonize()
-        returns False without doing anything. Otherwise, it sends
-        the application to the background, redirecting stdin/stdout
-        and changing its UID if requested.
+        Send the application to the background, redirecting
+        stdin/stdout and changing its UID if requested.
         """
-        if not self.params.daemonize:
-            return False
-
-        self.log.debug("Daemonizing")
         if os.fork(): sys.exit(0)
         os.umask(0) 
         os.setsid() 
@@ -577,7 +709,7 @@ class DaemonizingApp(LoggingApp):
         if self.params.pidfile:
             self.log.debug("Writing pidfile %s", self.params.pidfile)
             pidfile = open(self.params.pidfile, 'w')
-            pidfile.write('%i\n' % getpid())
+            pidfile.write('%i\n' % os.getpid())
             pidfile.close()
 
         if self.params.user:
@@ -591,8 +723,8 @@ class DaemonizingApp(LoggingApp):
             if delim != sep:
                 group = user
             self.log.debug("Changing to %s:%s", user, group)
-            os.setuid(pwd.getpwnam(user).pw_uid)
             os.setgid(grp.getgrnam(group).gr_gid)
+            os.setuid(pwd.getpwnam(user).pw_uid)
 
         self.log.debug("Changing directory to %s", self.chdir)
         os.chdir(self.chdir)
